/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * activetrx.p
 *
 *
 * Active trx monitoring (all TRX *except* "Allocated").
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	August 28, 2003
 *
 */

{lib/protop.i}
{lib/tick.i}

define output parameter dcDescription as character no-undo initial "ActiveTRX".

define temp-table tt_trx no-undo
  field xid       as integer   format ">>>>>>>>>9"    label "Id"
  field xvalid    as logical
  field usrnum    as integer   format ">>>>9"         label "Usr#"
  field userName  as character format "x(12)"         label "Name"
  field userPID   as character format "x(8)"          label "PID"
  field userFlags as character format "x(5)"          label "Flags"
  field userDev   as character format "x(12)"         label "Device"
  field trx-num   as integer   format ">>>>>>>>>9"    label "TRX#"
  field trx-rl    as integer   format ">>>>>>9"       label "BIClstr"
  field trx-st    as character format "x(4)"          label "Stat"
  field ztrxStart as character format "x(13)"         label "Start"
  field duraStr   as character format "x(10)"         label "  Duration"
  field idleStr   as character format "x(10)"         label "      Idle"
  field trx-wait  as character format "x(62)"         label "Wait Resource (dbkey)"
  field idleTime  as integer   format ">>>>9"         label "dbIdleSec"
  field duration  as integer   format ">>>>>9"        label "durLen"
  field zdbAccess as int64     format ">>>>>9"        label "logIOOps"

  index xid-idx is unique xid
  index duraStr-idx is primary duraStr descending
.

define new global shared variable rowLimit as integer no-undo.

procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_trx:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "usrnum",    ">>>9" ).
      altFormat( "80", b, "userName",  "x(14)" ).
      altFormat( "80", b, "userFlags", "x(5)" ).
      altFormat( "80", b, "duraStr",   "x(10)" ).
      altFormat( "80", b, "idleStr",   "x(10)" ).
      altFormat( "80", b, "trx-wait",  "x(31)" ).
    end.

  empty temp-table tt_trx.

  return.

end.

procedure mon-update:

  define input parameter argList as character no-undo.

  define variable wtblName as character no-undo.
  define variable wtblId   as integer   no-undo.
  define variable lineNum  as integer   no-undo.
  define variable procName as character no-undo.
  define variable dbIdle   as character no-undo.

  define variable i as integer no-undo.

  for each tt_trx:
    tt_trx.xvalid = no.
  end.

  run updTick.

  /* Get open transaction data from _Trans VST
   *
   */

  find dictdb._ActSummary no-lock.

  for each dictdb._Trans no-lock where _Trans-usrnum <> ? /* and _Trans-state <> "allocated" */ /* and _Trans-duration <> ? */ by _Trans-duration descending:

    if _Trans-duration >= _Summary-upTime then next.		/* every now and then a crazy duration appears */

    i = i + 1.

    assign
     wtblId   = ?
     wtblName = ""
    .

    find tt_trx where tt_trx.xid = _Trans-id no-error.
    if not available tt_trx then
      create tt_trx.

    assign
      tt_trx.xid      = _Trans-id
      tt_trx.xvalid   = yes
      tt_trx.ztrxStart = ( if _Trans-txtime <> ? then substring( _Trans-txtime, 12 ) else "" )
      tt_trx.usrnum   = _Trans-usrnum
      tt_trx.trx-num  = _Trans-num
      tt_trx.trx-rl   = _Trans-counter
      tt_trx.trx-st   = _Trans-state
      tt_trx.duration = ( if _Trans-duration <> ? then _Trans-duration else 0 )
      tt_trx.duraStr  = string( integer( truncate( tt_trx.duration / 86400, 0 )), ">" ) + " " + string(( tt_trx.duration modulo 86400 ), "hh:mm:ss" )
    .

    case tt_trx.trx-st:
      when "ALLOCATED"  then tt_trx.trx-st = "ALLO".
      when "ACTIVE"     then tt_trx.trx-st = "ACTV".
      when "BEGIN"      then tt_trx.trx-st = "BEG".
      when "DEAD"       then tt_trx.trx-st = "DEAD".
      when "COMMITTING" then tt_trx.trx-st = "COMM".
      when "PREPARING"  then tt_trx.trx-st = "PRPG".
      when "PREPARED"   then tt_trx.trx-st = "PRPD".
    end.

    /* get some info about the session that started the trx
     *
     */

    /* About usr#
     *
     * PROMON = usr#
     * .lg file = usr#
     * _myconnection.myconn-userid = usr#
     * _connection._connect-usr = usr#
     * _connection._connection-id = usr# + 1
     *
     * _connection-id is a *much* more efficient way to access _connect -- thus we
     * jump through hoops (add one to the usr#)
     *
     */

    find dictdb._Connect no-lock where _Connect-id = _Trans-usrnum + 1 no-error.

    /* _connect-wait2 is the table# (if it is a REC wait and the field exists)
     */

    if available _Connect and _Connect-wait = "REC" then
      do:
        wtblName = buffer _connect:handle:buffer-field( "_connect-wait2" ):buffer-value no-error.
        if wtblName <> "" and wtblName <> ? then							/* translate number to table name	*/
          do:
            wtblId = integer( wtblName ) no-error.
            find tt_tbl where tt_tbl.xid = wtblId no-error.
            if available tt_tbl then
              wtblName = tt_tbl.tblName.
          end.
      end.

    if available _Connect then
      assign
        tt_trx.userFlags = connectFlags( _Connect-Id )
        tt_trx.userName  = connectName( _Connect-Id, tt_trx.userFlags )
        tt_trx.userPID   = string( _connect-PID )
        tt_trx.userDev   = _connect-Device
        tt_trx.trx-wait  = substitute( "&1 &2 &3 &4", _connect-wait, _connect-wait1, wtblName, lastStatement( _connect-id, output lineNum, output procName, output dbIdle ))
      .

    /* keep track of how long it has been since we have seen any logical io activity
     */

    find dictdb._userio no-lock where _userio-id = ( _Trans-usrnum + 1 ) no-error.
    if available( _userio ) and ( _userio-dbAccess = tt_trx.zdbAccess ) then
      tt_trx.idleTime  = tt_trx.idleTime + z.
     else
      assign
        tt_trx.idleTime  = 0
        tt_trx.zdbAccess = _userio-dbAccess when available( _userio )
      .

    /* tt_trx.idleStr = string( tt_trx.idleTime, "hh:mm:ss" ). */
    tt_trx.idleStr = string( integer( truncate( tt_trx.idleTime / 86400, 0 )), ">" ) + " " + string(( tt_trx.idleTime modulo 86400 ), "hh:mm:ss" ).

    /* if ( rowLimit > 0 ) and ( i >= rowLimit ) then leave. */		/* need a sort order!	*/

  end.

  for each tt_trx where tt_trx.xvalid = no:
    delete tt_trx.
  end.

  add2ds( temp-table tt_trx:default-buffer-handle ).

  return.

end.

{ssg/activetrx.i}

return.
