/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * uio.p
 *
 *
 * UserIO monitoring.
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	September 5, 2003
 *
 */

{lib/protop.i}
{lib/tick.i}

define output parameter dcDescription as character no-undo initial "UserIOActivity".

{lib/tt_xstat.i}

define temp-table tt_uio no-undo
  field xId       as integer   format ">>>>9"      label "Id"
  field xValid    as logical
  field userNum   as integer   format ">>>>9"      label "Usr#"
  field userName  as character format "x(18)"      label "Name"
  field userPID   as character format "x(8)"       label "PID"
  field userFlags as character format "x(8)"       label "Flags"
  field dbAccess  as integer   format "->>>>>>9"   label "BlkAcc"
  field osReads   as integer   format "->>>>>>9"   label "OSRd"
  field osWrites  as integer   format "->>>>>9"    label "OSWr"
  field hitRatio  as decimal   format "->>9.99%"   label "Hit%"
  field recLks    as integer   format "->>>>>9"    label "RecLk"
/*field recWts    as integer   format "->>>>>9"    label "RecWts" */	/* replaced with Lk HWM -- much more useful	*/
  field lkHWM     as integer   format "->>>>>9"    label "LkHWM"
  field lastUpd   as character format "x(8)"       label " CSC Age"
  field lineNum   as integer   format "->>>>>"     label "Line#"
  field procName  as character format "x(50)"      label "Program Name"

  index xId-idx      is unique xid
  index dbAccess-idx is primary dbAccess descending
  index lkHWM-idx    lkHWM descending
  index recLks-idx   recLks descending
.

define new global shared variable rowLimit as integer no-undo.
define new global shared variable dbgMode  as integer no-undo initial 1.

define variable pt_trimStack as character no-undo	/* a list of items to automatically remove from the program stack	*/
  initial ""
.

subscribe to "enableCSC" anywhere run-procedure "enableCSC".

procedure enableCSC:

  define input parameter userNum as integer no-undo.
  define input parameter howLong as integer no-undo.

  publish "info" ( "enableCSC", substitute( "enableCSC: &1 &2 (not yet)", userNum, howLong )).

  return.

end.


function checkFilter returns logical ( connId as integer, filterList as character, f as character extent 32, p as character extent 32, nSpecial as integer, xtra as character ):

  define variable i   as integer   no-undo.
  define variable cc  as character no-undo.
  define variable cip as character no-undo.
  define variable cm  as character no-undo.
  define variable cn  as character no-undo.
  define variable ct  as character no-undo.
  define variable cb  as character no-undo.
  define variable cd  as character no-undo.

  define variable include as logical no-undo.

  include = no.								/* assume that the connection will not pass		*/

  find dictdb._Connect no-lock where _Connect-Id = connId no-error.
  if not available _connect then return ?.

  if filterList = "" or filterList = "*" then
    include = yes.							/* no filter, everyone passes!				*/
   else
    do:

      assign
        cb  = trim( _connect-batch )
        cd  = trim( _connect-device )
        ct  = trim( _connect-type )
        cn  = trim( _connect-name )
        cc  = ""
        cm  = ""
        cip = ""
      .

      cc =  buffer _connect:handle:buffer-field( "_connect-clientType" ):buffer-value no-error.
      cc = trim( cc ).

      cm =  buffer _connect:handle:buffer-field( "_connect-userMisc" ):buffer-value no-error.
      cm = trim( cm ).

      cip =  buffer _connect:handle:buffer-field( "_connect-IPAddress" ):buffer-value no-error.
      cip = trim( cip ).

      if lookup( "batch", filterList ) > 0 and ( ct matches "*batch*" or cb = "yes" ) and ( cc = "abl" ) then
        include = yes.

      if lookup( "self", filterList ) > 0 and ct = "self" then
        include = yes.

      if lookup( "remc", filterList ) > 0 and ct = "remc" then
        include = yes.

      if lookup( "brok", filterList ) > 0 then
        do:

          if ct = "brok" then						/* "the" db broker					*/
            include = yes.

          if lookup( ct, "serv,tsrv,sqsv" ) > 0 then			/* might be a login broker...				*/
            do:

              if index( cd, "batch" ) <= 0 then				/* kind of cheesy...					*/
                include = yes.

            end.

        end.

      if lookup( "serv", filterList ) > 0 and lookup( ct, "serv,tsrv,sqsv" ) > 0 then	/* exclude login brokers?	*/
        include = yes.

      if lookup( "helper", filterList ) > 0 and lookup( ct, "APW,BIW,AIW,WDOG,AIMD,RPLS,RPLA" ) > 0 then
        include = yes.

      if lookup( "util", filterList ) > 0 and lookup( ct, "MON,BKUP,SHUT,FMA,RPLU,RPLT,MVUT,DBAN" ) > 0 then
        include = yes.

      if lookup( "apsv", filterList ) > 0 and cc = "apsv" then
        include = yes.

      if lookup( "4gl", filterList ) > 0 and cc = "abl" then		/* 4gl as an alias for abl	*/
        include = yes.

      if lookup( "abl", filterList ) > 0 and cc = "abl" then
        include = yes.
        
      if lookup( "WS", filterList ) > 0 and cc = "wta" then
        include = yes.
        
      if lookup( "SQL", filterList ) > 0 and cc begins "SQ" then
        include = yes.
        
      if lookup( "PAS", filterList ) > 0 and cc begins "pas" then
        include = yes.

      if ( lookup( "pt3", filterList ) > 0 or lookup( "protop", filterList ) > 0 ) and cm begins "pt3:" then
        include = yes.

      /* if we have not already found an excuse to include a connection, try harder
       */

      if include = no and nSpecial > 0 then
        do:

          do i = 1 to nSpecial:

            if       f[i]      = "type"  and ( ct matches p[i] or cc matches p[i] ) then
              include = yes.
             else if f[i] begins "dev"   and cd  matches p[i] then
              include = yes.
             else if f[i] begins "name"  and cn  matches p[i] then
              include = yes.
             else if f[i] begins "ip"    and cip matches p[i] then
              include = yes.

          end.

        end.

    end.

/*
 *   if filterList <> "" then
 *     message
 *       "connId:" connId " type =" _Connect-type " filters =" filterList skip
 *       "[" nSpecial "]" f[1] "=" p[1] skip
 *       xtra include skip
 *      view-as
 *       alert-box information
 *     .
 */

  return include.

end.


/* initialize
 *
 */

procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_uio:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "userNum",   ">>>9" ).
      altFormat( "80", b, "userName",  "x(14)" ).
      altFormat( "80", b, "userFlags", "x(5)" ).
      altFormat( "80", b, "dbAccess",  ">>>>>>>9" ).
      altFormat( "80", b, "hitRatio",  ">>9.99%" ).
      altFormat( "80", b, "recLks",    ">>>>>9" ).
      altFormat( "80", b, "lineNum",   "->>>>9" ).
      altFormat( "80", b, "procName",  "x(23)" ).
    end.

  empty temp-table tt_xstat.

  run updTick.

  return.

end.

/* update
 *
 */


procedure mon-update:

  define input parameter argList as character no-undo.

  define variable i as integer no-undo.
  define variable j as integer no-undo.
  define variable n as integer no-undo.

  define variable fi as character no-undo extent 32.
  define variable pi as character no-undo extent 32.
  define variable ni as integer   no-undo.

  define variable fx as character no-undo extent 32.
  define variable px as character no-undo extent 32.
  define variable nx as integer   no-undo.

  run updTick.

  assign
    fi = ""
    pi = ""
    fx = ""
    px = ""
  .

  j = 0.
  n = num-entries( pt_connInclude ).
  do i = 1 to n:
    if num-entries( entry( i, pt_connInclude ), "=" ) = 2 then			/* field=pattern			*/
      assign
        j = j + 1
        fi[j] = entry( 1, entry( i, pt_connInclude ), "=" )
        pi[j] = entry( 2, entry( i, pt_connInclude ), "=" )
      .
    if j >= 32 then leave.
  end.
  ni = j.

  j = 0.
  n = num-entries( pt_connExclude ).
  do i = 1 to n:
    if num-entries( entry( i, pt_connExclude ), "=" ) = 2 then			/* field=pattern			*/
      assign
        j = j + 1
        fx[j] = entry( 1, entry( i, pt_connExclude ), "=" )
        px[j] = entry( 2, entry( i, pt_connExclude ), "=" )
      .
    if j >= 32 then leave.
  end.
  nx = j.


  for each tt_uio:
    xvalid = no.
  end.

  for each dictdb._UserIO no-lock where _UserIO-usr <> ?:

    find dictdb._Connect no-lock where _Connect-Id = _UserIO-Id.

    if checkFilter( _Connect-Id, pt_connInclude, fi, pi, ni, "include?" ) = no then next.

    if pt_ConnExclude <> "" then
      if checkFilter( _Connect-Id, pt_connExclude, fx, px, nx, "cull?" ) = yes then next.

    find tt_uio where tt_uio.xid = _UserIO-Id no-error.
    if not available( tt_uio ) then create tt_uio.
    assign
      tt_uio.xvalid    = yes
      tt_uio.xid       = _UserIO-Id
      tt_uio.userNum   = _UserIO-Usr
      tt_uio.userFlags = connectFlags( _Connect-Id )
      tt_uio.userName  = connectName( _Connect-Id, tt_uio.userFlags )
      tt_uio.userPID   = string( _connect-PID )
    .

    lastStatement( _Connect-Id, output tt_uio.lineNum, output tt_uio.procName, output tt_uio.lastUpd ).

    find dictdb._LockReq no-lock where _LockReq-Id = _UserIO-Id.

/*  tt_uio.tId  = buffer _userIO:handle:buffer-field( "_userIO-tenantId" ):buffer-value no-error.	*/

    run update_xstat (
      input _UserIO-Id,				/* unique key		*/
      input "", 				/* u_name		*/
      input "", 				/* u_flags		*/
      input "", 				/* u_pid		*/
      input "", 				/* string( _Connect-CacheLineNumber[1] ) + "," + _Connect-CacheInfo[1] */
      input "", 				/* _connect-clientType	*/
      input "",
      input _UserIO-dbaccess,			/* stat1		*/
      input _UserIO-dbread,			/* stat2		*/
      input _UserIO-dbwrite,			/* stat3		*/
      input 0,
      input _LockReq-RecLock,
      input _LockReq-RecWait,
      input 0,
      input 0
    ).

  end.

  run age_xstat.

  i = 0.
  for each tt_uio where tt_uio.xvalid = yes:

    find tt_xstat where tt_xstat.xid = tt_uio.xid no-error.
    if ( not available tt_xstat ) /* or ( tt_xstat.stat1[x] = 0 ) */ then
   /* or (( rowLimit > 0 ) and ( i >= rowLimit )) then */		/* this needs to be within the context of a sort order	*/
      do:
        tt_uio.xvalid = no.
        next.
      end.

    i = i + 1.

    assign
      tt_uio.xvalid    = yes
      tt_uio.dbAccess  = tt_xstat.stat1[x] / z
      tt_uio.OSReads   = tt_xstat.stat2[x] / z
      tt_uio.OSWrites  = tt_xstat.stat3[x] / z
      tt_uio.hitRatio  = tt_xstat.stat-ratio
      tt_uio.recLks    = tt_xstat.stat5[x] / z
/*    tt_uio.recWts    = tt_xstat.stat6[x] / z */
    .

    if pt_userlock = yes then
      do:
        find _userLock no-lock where _userLock-id =  tt_uio.xid.
        tt_uio.recLks = buffer _userLock:handle:buffer-field( "_userLock-Total" ):buffer-value no-error.
        tt_uio.lkHWM  = buffer _userLock:handle:buffer-field( "_userLock-HWM" ):buffer-value no-error.
      end.

  end.

  for each tt_uio where tt_uio.xvalid = no:
    delete tt_uio.
  end.

  j = 0.
  for each tt_uio where tt_uio.xvalid = yes:
    j = j + 1.
  end.

  /*
   *
  i = 0.
  if rowLimit > 0 then
    for each tt_uio by dbAccess:
      if i < rowLimit then
        i = i + 1.
       else
        delete tt_uio.
    end.
   *
   */

  if dbgMode >= 6 then message now "valid tt_uio rows:" j "  sent:" i.

  add2ds( temp-table tt_uio:default-buffer-handle ).

  return.

end.

{ssg/usract.i}
{ssg/usrlks.i}
{ssg/usrlkhwm.i}

{ssg/enablecsc.i}

return.
