/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * usersofidx.p
 *
 *
 * Users of an Index
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	September 5, 2003
 *
 */

{lib/protop.i}
{lib/tick.i}
{lib/tt_screenlist.i}

define output parameter dcDescription as character no-undo initial "UsersOfIndex".

{lib/tt_uindex.i}

define temp-table tt_uIdxAct no-undo
  field xid       as integer   format "->>>>9"
  field usrNum    as integer   format ">>>>9"        label "Usr#"
  field usrname   as character format "x(16)"        label "User Name"
  field idxRoot   as int64 format ">>>>>>>>>9"   label "Idx Root"
  field idxCr     as decimal   format ">>>>>>>>>9"   label "Create"
  field idxRd     as decimal   format ">>>>>>>>>>9"  label "Read"
  field idxSp     as decimal   format ">>>>>>>>>9"   label "Split"
  field idxDl     as decimal   format ">>>>>>>>>9"   label "Delete"
  field idxBlkDl  as decimal   format ">>>>9"        label "BlkDl"
  field lastUpd   as character format "x(8)"         label " CSC Age"
  field lineNum   as integer   format "->>>>"       label "Line#"
  field procName  as character format "x(55)"        label "Program Name"

  index xid-idx    is unique xid
  index idxRd-idx  is primary idxRd descending
.

define temp-table tt_uIdxAct_Info no-undo
  field infoString as character
  index infoString-idx is primary unique infoString
.

define new global shared variable rowLimit as integer no-undo.

define variable numIdxs as integer no-undo.
define variable numUsrs as integer no-undo.
define variable numUIdx as integer no-undo.
define variable idxNum  as integer no-undo.
define variable uIdxNum as integer no-undo initial ?.
define variable xIdxNum as integer no-undo initial ?.
define variable uIdxId  as integer no-undo initial ?.

define new global shared variable userIdxName as character   no-undo.			/* 							*/
define new global shared variable userIdxNum  as integer     no-undo.			/* 							*/


procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_uIdxAct:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "usrName",  "x(12)" ).
      altFormat( "80", b, "idxCr",    ">>>>>>9" ).
      altFormat( "80", b, "idxRd",    ">>>>>>9" ).
      altFormat( "80", b, "idxSp",    ">>>>>>9" ).
      altFormat( "80", b, "idxDl",    ">>>>>>9" ).
      altFormat( "80", b, "lineNum",  "->>>>>" ).
      altFormat( "80", b, "procName", "x(30)" ).
    end.

  find last _connect no-lock.
  find last _indexstat no-lock.
  
  assign
    numUsrs = integer( recid( _connect ))                                                       /* this is just showing off...          */
    numIdxs = recid( _indexstat )
    numUIdx = numUsrs * numIdxs
  .

  empty temp-table tt_uindex.

  run updTick.

  return.

end.


procedure mon-update:

  define input parameter argList as character no-undo.

  define variable i        as integer no-undo.
  define variable j        as integer no-undo.
  define variable k        as integer no-undo.

  define variable uIdxNum  as integer no-undo.
  define variable topLimit as integer no-undo initial 10.

  find tt_screenlist where tt_screenlist.dcName = dcDescription no-error.
  if not available( tt_screenlist ) or tt_screenlist.screenVisible = no then
    return.

  if argList begins "UIDXNUM" then
    do:
      uIdxNum = -999999.
      assign uIdxNum = integer( entry( 2, argList, "=" )) no-error.
    end.

  find first _userIndexStat no-lock where _userIndexStat-num = uIdxNum no-error.

  if not available( _userIndexStat ) then
    do:
       publish "protop_getUserIdxName" ( input-output userIdxName, input-output userIdxNum ).
       return.
    end.

  run updTick.

  i = uIdxNum.

  /* message "uIdxNum:" uIdxNum view-as alert-box information. */

  do while available( _userIndexStat ):
    run upd-tt_uindex ( _UserIndexStat._UserIndexStat-id, uIdxNum, ? ).
    i = _userIndexStat-id + numIdxs.
    find _userIndexStat no-lock where _userIndexStat-id = i no-error.
  end.

  /* run age_index. */

  empty temp-table tt_uIdxAct.
  empty temp-table tt_uIdxAct_Info.

  /* getTableInfo( tt_utable.tblName, ... ). ???  */

  create tt_uIdxAct_Info.
  tt_uIdxAct_Info.infoString = substitute( 'Users of "&1" &2 idxNum: &3  areaNum: &4  idxRoot: &5', tt_uindex.idxName, tt_uindex.idxNote, tt_uindex.idxNum, tt_uindex.areaNum, tt_uindex.idxRoot ).

  /* top X creates
   */

  i = 0.
  for each tt_uindex no-lock where tt_uindex.idx-cre[x] > 0 by tt_uindex.idx-cre[x] descending.

    i = i + 1.

    find tt_uIdxAct where tt_uIdxAct.usrNum = tt_uindex.usrNum no-error.		/* prevent duplicates			*/
    if available tt_uIdxAct then next.

    create tt_uIdxAct.
    assign
      tt_uIdxAct.xid       = tt_uindex.usrNum
      tt_uIdxAct.usrNum    = tt_uindex.usrNum
      tt_uIdxAct.usrName   = tt_uindex.usrName
      tt_uIdxAct.idxRoot   = tt_uindex.idxRoot
      tt_uIdxAct.idxCr     = ( tt_uindex.idx-cre[x]    / z )
      tt_uIdxAct.idxRd     = ( tt_uindex.idx-rd[x]     / z )
      tt_uIdxAct.idxSp     = ( tt_uindex.idx-split[x]  / z )
      tt_uIdxAct.idxDl     = ( tt_uindex.idx-del[x]    / z )
      tt_uIdxAct.idxBlkDl  = ( tt_uindex.idx-blkdel[x] / z )
    .
    lastStatement( tt_uindex.usrNum + 1, output tt_uIdxAct.lineNum, output tt_uIdxAct.procName, tt_uIdxAct.lastUpd ).

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X reads
   */

  i = 0.
  for each tt_uindex no-lock where tt_uindex.idx-rd[x] > 0 by tt_uindex.idx-rd[x] descending.

    i = i + 1.

    find tt_uIdxAct where tt_uIdxAct.usrNum = tt_uindex.usrNum no-error.		/* prevent duplicates			*/
    if available tt_uIdxAct then next.

    create tt_uIdxAct.
    assign
      tt_uIdxAct.xid       = tt_uindex.usrNum
      tt_uIdxAct.usrNum    = tt_uindex.usrNum
      tt_uIdxAct.usrName   = tt_uindex.usrName
      tt_uIdxAct.idxRoot   = tt_uindex.idxRoot
      tt_uIdxAct.idxCr     = ( tt_uindex.idx-cre[x]    / z )
      tt_uIdxAct.idxRd     = ( tt_uindex.idx-rd[x]     / z )
      tt_uIdxAct.idxSp     = ( tt_uindex.idx-split[x]  / z )
      tt_uIdxAct.idxDl     = ( tt_uindex.idx-del[x]    / z )
      tt_uIdxAct.idxBlkDl  = ( tt_uindex.idx-blkdel[x] / z )
    .
    lastStatement( tt_uindex.usrNum + 1, output tt_uIdxAct.lineNum, output tt_uIdxAct.procName, tt_uIdxAct.lastUpd ).

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X splits
   */

  i = 0.
  for each tt_uindex no-lock where tt_uindex.idx-split[x] > 0 by tt_uindex.idx-split[x] descending.

    i = i + 1.

    find tt_uIdxAct where tt_uIdxAct.usrNum = tt_uindex.usrNum no-error.		/* prevent duplicates			*/
    if available tt_uIdxAct then next.

    create tt_uIdxAct.
    assign
      tt_uIdxAct.xid       = tt_uindex.usrNum
      tt_uIdxAct.usrNum    = tt_uindex.usrNum
      tt_uIdxAct.usrName   = tt_uindex.usrName
      tt_uIdxAct.idxRoot   = tt_uindex.idxRoot
      tt_uIdxAct.idxCr     = ( tt_uindex.idx-cre[x]    / z )
      tt_uIdxAct.idxRd     = ( tt_uindex.idx-rd[x]     / z )
      tt_uIdxAct.idxSp     = ( tt_uindex.idx-split[x]  / z )
      tt_uIdxAct.idxDl     = ( tt_uindex.idx-del[x]    / z )
      tt_uIdxAct.idxBlkDl  = ( tt_uindex.idx-blkdel[x] / z )
    .
    lastStatement( tt_uindex.usrNum + 1, output tt_uIdxAct.lineNum, output tt_uIdxAct.procName, tt_uIdxAct.lastUpd ).

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X deletes
   */

  i = 0.
  for each tt_uindex no-lock where tt_uindex.idx-del[x] > 0 by tt_uindex.idx-del[x] descending.

    i = i + 1.

    find tt_uIdxAct where tt_uIdxAct.usrNum = tt_uindex.usrNum no-error.		/* prevent duplicates			*/
    if available tt_uIdxAct then next.

    create tt_uIdxAct.
    assign
      tt_uIdxAct.xid       = tt_uindex.usrNum
      tt_uIdxAct.usrNum    = tt_uindex.usrNum
      tt_uIdxAct.usrName   = tt_uindex.usrName
      tt_uIdxAct.idxRoot   = tt_uindex.idxRoot
      tt_uIdxAct.idxCr     = ( tt_uindex.idx-cre[x]    / z )
      tt_uIdxAct.idxRd     = ( tt_uindex.idx-rd[x]     / z )
      tt_uIdxAct.idxSp     = ( tt_uindex.idx-split[x]  / z )
      tt_uIdxAct.idxDl     = ( tt_uindex.idx-del[x]    / z )
      tt_uIdxAct.idxBlkDl  = ( tt_uindex.idx-blkdel[x] / z )
    .
    lastStatement( tt_uindex.usrNum + 1, output tt_uIdxAct.lineNum, output tt_uIdxAct.procName, tt_uIdxAct.lastUpd ).

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X block deletes
   */

  i = 0.
  for each tt_uindex no-lock where tt_uindex.idx-blkdel[x] > 0 by tt_uindex.idx-blkdel[x] descending.

    i = i + 1.

    find tt_uIdxAct where tt_uIdxAct.usrNum = tt_uindex.usrNum no-error.		/* prevent duplicates			*/
    if available tt_uIdxAct then next.

    create tt_uIdxAct.
    assign
      tt_uIdxAct.xid       = tt_uindex.usrNum
      tt_uIdxAct.usrNum    = tt_uindex.usrNum
      tt_uIdxAct.usrName   = tt_uindex.usrName
      tt_uIdxAct.idxRoot   = tt_uindex.idxRoot
      tt_uIdxAct.idxCr     = ( tt_uindex.idx-cre[x]    / z )
      tt_uIdxAct.idxRd     = ( tt_uindex.idx-rd[x]     / z )
      tt_uIdxAct.idxSp     = ( tt_uindex.idx-split[x]  / z )
      tt_uIdxAct.idxDl     = ( tt_uindex.idx-del[x]    / z )
      tt_uIdxAct.idxBlkDl  = ( tt_uindex.idx-blkdel[x] / z )
    .
    lastStatement( tt_uindex.usrNum + 1, output tt_uIdxAct.lineNum, output tt_uIdxAct.procName, tt_uIdxAct.lastUpd ).

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* message tt_uIdxAct_Info.infoString view-as alert-box. */

  add2ds( temp-table tt_uIdxAct:default-buffer-handle ).
  add2ds( temp-table tt_uIdxAct_Info:default-buffer-handle ).

  return.

end.

return.
