/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2006 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * tt_lobdata.i
 *
 */

define temp-table tt_lobdata no-undo
  field xid      as integer
  field xvalid   as logical
  field LOBnum   as integer
  field areaNum  as integer
  field LOBPool  as character
  field LOBname  as character label "LOB Name" format "x(30)"
  field TblNm    as character label "Table Name" format "x(30)"
  field LOB-cre  as decimal extent 5 label "Create"  format ">>>>>>>>>>>>>>>>9"
  field LOB-rd   as decimal extent 5 label "Read"    format ">>>>>>>>>>>>>>>>9"
  field LOB-upd  as decimal extent 5 label "Update"  format ">>>>>>>>>>>>>>>>9"
  field LOB-del  as decimal extent 5 label "Delete"  format ">>>>>>>>>>>>>>>>9"
  field LOB-osrd as decimal extent 5 label "OS Read" format ">>>>>>>>>>>>>>>>9"
  field pct-cre  as decimal   label "%" format ">>9.99"
  field pct-rd   as decimal   label "%" format ">>9.99"
  field pct-upd  as decimal   label "%" format ">>9.99"
  field pct-del  as decimal   label "%" format ">>9.99"
  index xid-idx is unique primary xid.

procedure upd-tt_lobdata:

  define input parameter p_LOB as integer no-undo.

  define variable x as integer no-undo.
  define variable z as integer no-undo.

  define variable bh  as handle no-undo.
  define variable bf  as handle no-undo.

  do-SumSample( output x, output z ).

&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) >= 12.0 &THEN

  find dictdb._LOBStat no-lock where _LOBStat-id = p_LOB no-error.

  find tt_lob no-lock where tt_lob.xid = p_LOB no-error.
  if not available tt_lob then return.

  find tt_lobdata exclusive-lock where tt_lobdata.xid = p_LOB no-error.

  if available tt_lobdata then
    tt_lobdata.xvalid = yes.
   else
    do:
      create tt_lobdata.
      assign
        tt_lobdata.xid      = p_LOB
        tt_lobdata.xvalid   = yes
        tt_lobdata.lobNum   = tt_lob.xid
        tt_lobdata.lobName  = tt_lob.lobname
        tt_lobdata.areanum  = tt_lob.areanum
        tt_lobdata.lobPool  = tt_lob.lobPool
      .

      find tt_tbl where tt_tbl.xid = tt_lob.tblId no-error.
      if available tt_tbl then
        tt_lobdata.tblNm = tt_tbl.tblname.

      if available _LOBStat then		/* _LOBStat range may not be active...	*/
        do:
          assign
            {lib/init-xrec.i tt_lobdata.lob-cre  _LOBStat-create}
            {lib/init-xrec.i tt_lobdata.lob-rd   _LOBStat-read}
            {lib/init-xrec.i tt_lobdata.lob-upd  _LOBStat-update}
            {lib/init-xrec.i tt_lobdata.lob-del  _LOBStat-delete}
            {lib/init-xrec.i tt_lobdata.lob-osrd _LOBStat-OSRead}
          .
        end.
    end.

  if available _LOBStat then		/* _LOBStat range may not be active...	*/
    do:

      assign
        {lib/upd-xrec.i tt_lobdata.lob-cre  _LOBStat-create}
        {lib/upd-xrec.i tt_lobdata.lob-rd   _LOBStat-read}
        {lib/upd-xrec.i tt_lobdata.lob-upd  _LOBStat-update}
        {lib/upd-xrec.i tt_lobdata.lob-del  _LOBStat-delete}
        {lib/upd-xrec.i tt_lobdata.lob-osrd _LOBStat-OSRead}
      .

    end.

&ENDIF

  return.

end.

/* this will be called by the including procedure's mon-update procedure
 *
 */

procedure age_table:

  for each tt_lobdata exclusive-lock:

    if tt_lobdata.xvalid = no then
      delete tt_lobdata.
     else
      assign
        tt_lobdata.xvalid = no
      .

  end.

  return.

end.
