/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2006 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * vstlib.p
 *
 * Functions that deal with VSTs
 *
 *
 * Known Bugs & Issues:
 *
 *
 * To Do:
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	August 28, 2003
 *
 *
 * History:
 *
 *	Accepted changes from Sam Paakki regarding "quit" function and PROPATH
 *	September 26, 2003
 *
 *      Accepted changes from Patrick Tingen to set html output dir,   
 *      eliminating the nasty curr-page shared variable, simplifying the
 *	release# and adding protop-url 
 *      October 30, 2003
 * 
 */

{lib/protop.i}

define new global shared temp-table tt_lock no-undo like _lock
  field qflag as logical
  index id-idx is unique primary _lock-id
  index recid-idx _lock-recid
  index usr-idx _lock-usr
  index tbl-idx _lock-table
.


/*** Install self as a session super-procedure
 ***
 ***/

session:add-super-procedure( this-procedure ).

define variable i                  as integer no-undo.
define variable hasClstrSz         as logical no-undo.
define variable dbRecid            as recid   no-undo.
define variable isMultiTenantTable as logical no-undo.
define variable bhFile             as handle  no-undo.
define variable fileQueryOK        as logical no-undo.

find first dictdb._DB no-lock where _Db._Db-local.
dbRecid = recid( _DB ).

hasClstrSz = no.

find dictdb._File no-lock where _File-Name = "_Area" no-error.

if available _file then
  do:
    find dictdb._Field no-lock where _Field._File-recid = recid( _File ) and _Field-Name = "_Area-ClusterSize" no-error.
    if available _Field then hasClstrSz = yes.
  end.
 else
  do:
    message now "lib/vstlib.p: _Area is not a known table - something is seriously wrong.".
  end.

run chkStartup.

/*** initialize tt cache for various schema tables
 ***
 ***/

empty temp-table tt_lob.
empty temp-table tt_tbl.
empty temp-table tt_idx.
empty temp-table tt_areaExtent.
empty temp-table tt_area.

for each dictdb._storageObject no-lock where _storageObject._Db-recid = dbRecid and _Object-type = 3:

  find _file  no-lock where _file-num = _Object-associate no-error.
  if not available _file then next.

  find _field no-lock where _file-recid = recid( _file ) and _fld-stlen = _Object-number no-error.
  if not available _field then next.

  find _area  no-lock where _area._area-number = _storageObject._Area-Number no-error.
  if not available _area then next.

  find tt_lob where tt_lob.xid =  _Object-number no-error.
  if not available tt_lob then
    do:
      create tt_lob.
      assign
        tt_lob.xid     = _Object-number
        tt_lob.tblId   = _Object-associate
        tt_lob.tblName = _file-name
        tt_lob.LOBName = _field-name
        tt_lob.areaNum = _storageObject._Area-Number when available _storageObject
        tt_lob.LOBPool = "b1"	/* default */
        tt_lob.LOBPool = ( if max( get-bits( _object-attrib, 7, 1 ), get-bits( _area-attrib, 7, 1 )) = 0 then "B1" else "B2" ) when available _storageObject
      no-error.
    end.

  /*
   *  message tt_lob.xid tt_lob.tblName tt_lob.LOBName tt_lob.areaNum tt_lob.LOBPool.
   *  pause.
   */

end.


/* new meta-schema caching for multi-tenancy compatibility */

create buffer bhFile for table "dictdb._File".

for each dictdb._File no-lock:

  find first tt_tbl where tt_tbl.tblName = _file._file-name no-error.
  if available tt_tbl then next.						/* don't build the caches twice...		*/

  find first dictdb._storageObject no-lock 
    where _storageObject._Db-recid = dbRecid
      and _Object-type             = 1
      and _Object-number           = _file._file-num no-error.

  if not available _storageObject then next.

  find _area no-lock where _area._area-number = _storageObject._Area-Number no-error.

  fileQueryOK        = bhfile:find-by-rowid( rowid( _File ), no-lock ).                             /* _file._file-attributes[1] = yes when the table is multi-tenant                     */
  isMultiTenantTable = bhFile:buffer-field( "_file-attributes" ):buffer-value( 1 ) no-error.        /* isMultiTenantTable = ? if the field _file-attributes doesn't exist (i.e. pre-11.0) */
  
  if isMultiTenantTable = ? then isMultiTenantTable = no.                                           /* pre-11.0, so the table must be shared (non-multi-tenant) */

  create tt_tbl no-error.
  assign
    tt_tbl.xid           = _File._File-num
    tt_tbl.areaNum       = _storageObject._Area-Number when available _storageObject
    tt_tbl.tblPool       = "b1"	/* default */
    tt_tbl.tblPool       = ( if max( get-bits( _storageObject._object-attrib, 7, 1 ), get-bits( _area._area-attrib, 7, 1 )) = 0 then "B1" else "B2" ) 
                           when ( available _storageObject and available _area )
    tt_tbl.tblname       = _File._File-name
    tt_tbl.isMultiTenant = isMultiTenantTable   
  no-error.

  for each dictdb._Index no-lock of dictdb._File:

    find first _storageObject no-lock
      where _storageObject._Db-recid = dbRecid
        and _Object-type             = 2
        and _Object-number           = _index._idx-num no-error.
    if not available _storageObject then next.

    find _area no-lock where _area._area-number = _storageObject._Area-Number no-error.

    create tt_idx no-error.
    assign
      tt_idx.xid     = _Index._Idx-num
      tt_idx.idxname = _Index._Index-name
      tt_idx.tblnum  = _File._File-num when available _file
      tt_idx.tblname = _File._File-name when available _file
      tt_idx.idxnote = ( if dictdb._file._prime-index = recid( _index ) then "P" else "" ) +
                       ( if _index._unique then "U" else "" )
      tt_idx.idxRoot = int64( _Object-root ) when available _storageObject
      tt_idx.areaNum = int64( _storageObject._Area-Number ) when available _storageObject
      tt_idx.idxPool = "b1"	/* default */
      tt_idx.idxPool = ( if max( get-bits( _storageObject._object-attrib, 7, 1 ), get-bits( _area._area-attrib, 7, 1 )) = 0 then "B1" else "B2" ) 
                       when ( available _storageObject and available _area )
    no-error.

  end.

end.

/* clean up dynamic buffer */
delete object bhFile.
bhFile = ?.


for each dictdb._areaExtent no-lock:

  find tt_areaExtent
    where tt_areaExtent.areaNum = _areaExtent._Area-Number and tt_areaExtent.extNum  = _areaExtent._Extent-Number
    no-error.
  if available tt_areaExtent then next.						/* don't build the caches twice...		*/

  create tt_areaExtent.
  assign
    tt_areaExtent.areaNum = _areaExtent._Area-Number
    tt_areaExtent.extNum  = _areaExtent._Extent-Number
    tt_areaExtent.extSize = _areaExtent._Extent-Size
    tt_areaExtent.extType = _areaExtent._Extent-Type
    tt_areaExtent.extPath = _areaExtent._Extent-Path
  .

end.

for each dictdb._Area no-lock:

  /* if _Area._Area-number <= 3 or _Area._Area-type = 7 then next. */		/* skip control area, bi area and after image areas		*/
  find tt_area where tt_area.SANum = _Area._Area-Number no-error.

  if not available tt_area then
    do:

      find first _AreaStatus no-lock where _AreaStatus-areaNum = _Area._Area-Number no-error.
      if not available _AreaStatus then next.

      create tt_area.
      assign
        tt_area.xid      = _AreaStatus._AreaStatus-Id		/* need an xid field for context	*/
        tt_area.SANum    = _Area._Area-Number
        tt_area.areaPool = "b1"		/* default */
        tt_area.areaPool = ( if get-bits( _area-attrib, 7, 1 ) = 0 then "B1" else "B2" )
        tt_area.SAName   = string( _Area._Area-Number )		/* temporary, will be populated below	*/
      .

    end.

end.

/* end caching schema tables
 *
 */


run lib/parsedba.p persistent.
run lib/getdba.p persistent.


/*** Prior to 11.7 _TableStat-Id mapping to _File-num is "quirky" when _statbase._tablebase <> 1, ditto for _Indexstat
 ***
 *** Calculating and applying the offsets is now handled in dc/tblact.p and dc/idxact.p
 ***
 *
 * find _tablestat no-lock where _tablestat-id = 1.
 *
 * display _tablestat.	/* prior to 11.7 _tablestat-id will magically change to whatever table is mapped to slot #1	*/
 * pause.
 *
 * if _tablestat-id <> 1 then
 *   i = ( _tablestat-id * -1 ) + 1.	/* this is the offset that needs to be applied to real table numbers to map to _tablestat	*/
 *
 * find _tablestat no-lock where _tablestat-id = ( i + 1 ).
 * display _tablestat.
 * pause.
 *
 ***
 ***/


return.



/** VST functions
 **
 **/

function isDBFeatureActive returns logical ( featureId as int64 ):

  /* given a database feature id, return true if it exists and is active, or false if either it is inactive or does not exist */
  
  find dictdb._database-feature no-lock where _database-feature._dbfeature-id = featureId no-error.
  
  if available( _database-feature ) and _database-feature._dbfeature_active = '1' then
    return true.
  else
    return false.

end function.

function connectFlags returns character( cnxId as integer ):

  define variable u_flags as character no-undo.
  define variable xflag   as character no-undo.
  define variable cc      as character no-undo.
  define variable um      as character no-undo.
  define variable cn      as character no-undo.

  find dictdb._Connect no-lock where _Connect-Id = cnxId no-error.
  if not available _connect then return "".

  if       _Connect-type = "self" then					/* shared memory	*/
    u_flags = "S".
   else if _Connect-type = "remc" then					/* remote client	*/
    u_flags = "R".
   else if _Connect-type = "brok" then					/* login broker		*/
    u_flags = "L".
   else if lookup( trim( _connect-type ), "serv,tsrv,sqsv" ) > 0 then	/* client/server server	*/
    u_flags = "@".
   else if lookup( trim( _connect-type ), "APW,BIW,AIW,WDOG" ) > 0 then	/* "helper"		*/
    u_flags = "H".
   else
    u_flags = "O".			/* "Other" -- known uncategorized:  rpls,rpla,aimd	*/

  cc = "".
  cc =  buffer _connect:handle:buffer-field( "_connect-clientType" ):buffer-value no-error.

  if       cc = "abl" then      u_flags = u_flags + "4".		/* 4GL client		*/
   else if cc = "wta" then      u_flags = u_flags + "W".		/* Webspeed		*/
   else if cc = "apsv" then     u_flags = u_flags + "A".		/* Classic Appserver	*/
   else if cc begins "pas" then u_flags = u_flags + "P".		/* PASN and PASA	*/
   else if cc begins "SQ"  then u_flags = u_flags + "Q".		/* SQLC and SQFC	*/
   else                         u_flags = u_flags + "X".		/* Mystery client type!	*/

  /* q: why, why is it so difficult to determine if a session is a batch session?
   * a: nobody knows, it just is!
   *
   * q: why is _connect-batch a /character/ field?!?
   * a: nobody knows, it just is!
   *
   * q: what is the '_connect-device matches "*batch*"' logic seen here and there all about?
   * a: sometimes _connect-device = host + " batch" or host + " ###"
   *
   */

  if _Connect-device matches "*batch*" or _connect-batch = "yes" then
    u_flags = u_flags + "B".
   else
    u_flags = u_flags + " ".	/* makes for nicer alignment	*/

  /* identify the current process as ProTop 3
   */

  find _myconnection no-lock.
  if _myconn-pid = _connect-pid then u_flags = "PT3".

  um = "".
  um =  buffer _connect:handle:buffer-field( "_connect-usermisc" ):buffer-value no-error.
  if um begins "pt3:" then
    u_flags = "PT3".

  xflag = "".
  if _Connect-TransId    > 0 then xflag = xflag + "*".		/* transaction is active			*/
  if _Connect-Disconnect = 1 then xflag = xflag + "d".		/* disconnect in process			*/
  if _Connect-Resync     = 1 then xflag = xflag + "r".		/* error, 4gl undo & trx backout or disconnect	*/ 
  if _Connect-Interrupt  = 1 then xflag = xflag + "i".		/* control-c or STOP				*/

  /* new oe12 "notification" fields need to be added
   *
   * -:-	no notifications
   * R:-	recache marked but not yet sent to user
   * R:R	recache both pending and being processed
   * -:R	no recache pending, previous recache being processed
   *
   * (xflag removes the ":" to save space)
   *
   */

  cn = "".
  cn =  buffer _connect:handle:buffer-field( "_connect-notifications" ):buffer-value no-error.
  if cn <> "-:-" then
    assign
      cn    = replace( cn, "R", "N" )
      xflag = xflag + replace( cn, ":", "" )
    .

  return u_flags + xflag.

end.

function connectName returns character( cnxId as integer, cnxFlags as character ):

  define variable u_name  as character no-undo.
  define variable cc      as character no-undo.
  define variable um      as character no-undo.

  find dictdb._Connect no-lock where _Connect-Id = cnxId no-error.
  if not available _connect then return "".

  if cnxFlags begins "L" or cnxFlags begins "O" or cnxFlags begins "H" or cnxFlags begins "@" then
    u_name = trim( substitute( "&1 &2", _connect-name, _connect-type )).
   else
    u_name = _Connect-name.

  /* this was getting way too complicated and confusing -- so simply go with: use device name unless device = "batch"
   */

  if u_name = "" and _connect-device <> "batch" then
    u_name = entry( 1, _connect-device, " " ).

  um = "".
  um =  buffer _connect:handle:buffer-field( "_connect-usermisc" ):buffer-value no-error.
  if um begins "pt3:" then
    u_name = trim( substitute( "&1 &2", entry( 2, um, ":" ), u_name )).

  /* if there is still no name, use the usr# (login with blank id)
   *
   * used to be _connect-PID but that is less useful than usr#
   *
   */

  return ( if u_name = "" then string( _connect-usr ) else u_name ).

end.

function lastStatement returns character( cnxId as integer, output lineNum as integer, output procName as character, output lastUpdStr as character ):

  define variable ct as character no-undo.
  define variable ci as character no-undo.
  define variable cl as integer   no-undo.
  define variable cz as character no-undo initial ?.

  define variable lastUpd    as datetime  no-undo.
  define variable lastUpdInt as integer   no-undo.

  define variable ml as character no-undo initial "Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec".

  find dictdb._Connect no-lock where _Connect-Id = cnxId no-error.
  if not available _connect then return "".

  ct =  buffer _connect:handle:buffer-field( "_connect-clientType" ):buffer-value no-error.
  ci =  buffer _connect:handle:buffer-field( "_connect-cacheInfo" ):buffer-value( 1 ) no-error.
  cl =  buffer _connect:handle:buffer-field( "_connect-cacheLineNumber" ):buffer-value( 1 ) no-error.
  cz =  buffer _connect:handle:buffer-field( "_connect-cacheLastUpdate" ):buffer-value() no-error.

  assign
    procName  = ( if ci = ? then "" else ci )
    lineNum   = ( if cl = ? then 0  else cl )
  .

  /* Note: lineNum -1 = "cleanup" at the end of a procedure.	*/

  /* eliminate optional internal procedure name if it is not a SQL connection
   *
   * IPName = trim( entry( 1, _Connect-CacheInfo[1], " " ))
   *
   */

  if not ( ct begins "SQ" ) and num-entries( ci, " " ) > 1 then
    assign
      procName  = trim( entry( 2, ci, " " ))
    .

  /* sql statements can have formatting that is "unhelpful" in the context
   * of ProTop so flatten that out and make the output more readable
   */

  procName = replace( procName, "~n", " " ).
  procName = replace( procName, "~r", " " ).
  procName = replace( procName, "~\", "|" ).
  procName = replace( procName, "~f", " " ).
  procName = replace( procName, "~t", " " ).
  do while index( procName, "  " ) > 0:
    procName = replace( procName, "  ", " " ).
  end.

  if cz <> ? then
    do:

      lastUpd    = datetime( fixDateString( cz )) no-error.
      lastUpdInt = interval( now, lastUpd, "seconds" ) no-error.

      if year( now ) <> year( lastUpd ) then
        lastUpdStr = substitute( "&1 &2", entry( month( lastUpd ), ml ), string( year( lastUpd ), "9999" )).
       else if lastUpdInt > 86400 then
        lastUpdStr = substitute( "&1 &2", entry( month( lastUpd ), ml ), string( day( lastUpd ), "99" )).
       else
        lastUpdStr = string( lastUpdInt, "hh:mm:ss" ).

    end.

  return ( if ct begins "SQ" then procName else substitute( "&1 &2", lineNum, procName )).

end.


/* Thanks to George Potemkin!
 *
 */

function aiInfo returns character ( input vAiFile as character, output vAiGenNum as integer ):

  DEFINE VARIABLE vAiBlkSize AS INTEGER   NO-UNDO.

  DEFINE VARIABLE vAiStatus  AS INTEGER   NO-UNDO.
  DEFINE VARIABLE vStatus    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE vRaw       AS RAW       NO-UNDO.

  define variable aiSeqNum as int64     initial ?.
  define variable aiStatus as character initial ?.

  for each _areaStatus no-lock where _areaStatus-areaName begins "after image area":

    if vAiFile = _areaStatus-lastExtent then
      do:
        aiSeqNum = ( buffer _areaStatus:handle:buffer-field( "_areaStatus-AI-Seq" ):buffer-value ) no-error.
        aiStatus = ( buffer _areaStatus:handle:buffer-field( "_areaStatus-State" ):buffer-value )  no-error.
      end.

  end.

  if aiSeqNum <> ? then
    do:
      vAiGenNum = aiSeqNum.
      return aiStatus.
    end.

  /*** old style ***/

  find first _logging no-lock no-error.
  if not available _logging then return "".

  vAiBlkSize = _logging._logging-aiBlkSize.

  file-info:file-name = vAiFile.

  if file-info:full-pathname <> ? and index( file-info:file-type, "r" ) <> 0  and ( vAiBlkSize > 0 ) then
    do:

      /* Offset Len  Description
       * ------ ---  -----------
       * 24-27    4  AiGenNum      AI extent sequence number (mb_aigennbr, Seqno)
       * 40-43    4  AiStatus      1=Busy, 2=Empty, 4=Full
       *
       * v10 moved everything 65 bytes...
       * ( if integer( dbversion(1)) > 9 then 65 else 0 ).
       */

      INPUT stream inStrm FROM VALUE( vAiFile ).

      ASSIGN LENGTH( vRaw ) = 4.
      SEEK stream inStrm TO 24 + vAiBlkSize + ( if integer( dbversion(1)) > 9 then 64 else 0 ).
      IMPORT  stream inStrm UNFORMATTED vRaw.
      ASSIGN vAiGenNum = GET-LONG( vRaw, 1 ).
            
      SEEK  stream inStrm TO 40 + vAiBlkSize + ( if integer( dbversion(1)) > 9 then 64 else 0 ).
      IMPORT stream inStrm UNFORMATTED vRaw.
      ASSIGN vAiStatus = GET-LONG(vRaw, 1).

      ASSIGN LENGTH(vRaw) = 0.
      INPUT stream inStrm CLOSE.

      vStatus =
        IF       vAiStatus EQ 0 THEN "None"		/* Msg 694	*/
         ELSE IF vAiStatus EQ 1 THEN "Busy"		/* Msg 3792	*/
         ELSE IF vAiStatus EQ 2 THEN "Empty"		/* Msg 3793	*/
         ELSE IF vAiStatus EQ 4 THEN "Full"		/* Msg 3794	*/
         ELSE IF vAiStatus EQ 8 THEN "Locked"		/* ????		*/
         ELSE "Unknown".				/* Msg 737	*/

    end.
   else
    do:
      vStatus = "Can not open".
    end.

  return vstatus.

end.	/* aiInfo */


/* get consolidated info about a storage area
 *
 */

procedure initAreaInfo:

  define variable seqnum     as integer no-undo.

  define variable area_bh    as handle  no-undo.
  define variable area_bf    as handle  no-undo.

  for each dictdb._areaStatus no-lock:

/*
 *  if    _AreaStatus-AreaName begins "Control Area"
 *     or _AreaStatus-AreaName begins "Primary Recovery Area"
 *     or _AreaStatus-AreaName begins "After Image Area" then next.
 */

    find tt_area where tt_area.SANum = _AreaStatus-AreaNum.

    find dictdb._Area no-lock where _Area._Area-number = tt_area.SANum no-error.
    if not available _Area then next.

    assign
      tt_area.SAName        = _AreaStatus-Areaname
  /*  tt_area.areaStatus-Id = _areaStatus-id */
      tt_area.rpb           = exp( 2, _Area._Area-recbits )			/* thanks to Dmitri Levin!			*/
      tt_area.numExts       = _AreaStatus-Extents
    .

    /* count the number of storage objects (tables & indexes) in this storage area
     */

    assign
      tt_area.numTbls = 0
      tt_area.numIdxs = 0
      tt_area.numLOBs = 0
      tt_area.xnote = ""
    .

    for each _storageobject no-lock where _StorageObject._DB-Recid = dbRecid
         and _storageobject._area-number = tt_area.SANum
         and _storageobject._object-num > 0
         and _storageobject._object-associate > 0:

      if       _storageobject._object-type = 1 then tt_area.numTbls = tt_area.numTbls + 1.
       else if _storageobject._object-type = 2 then tt_area.numIdxs = tt_area.numIdxs + 1.
       else if _storageobject._object-type = 3 then tt_area.numLOBs = tt_area.numLOBs + 1.

    end.

    /* if there are data tables or indexes in the schema area (area 6) note it with a "*"
     */

    if tt_area.SANum = 6 and ( tt_area.numTbls + tt_area.numIdxs + tt_area.numLOBs ) > 0 then tt_area.xnote = tt_area.xnote + "*".

    if _areaStatus-Areaname begins "After Image Area" then
      do:
        tt_area.xNote = ?.
        tt_area.xNote = ( buffer _areaStatus:handle:buffer-field( "_areaStatus-AI-Seq" ):buffer-value ) no-error.
        if tt_area.xNote = ? then
          tt_area.xNote = aiInfo( input _AreaStatus-LastExtent, output seqnum ).
      end.

    if hasClstrSz = false then
      tt_area.clstrSz = 0.
     else
      do:
        assign        
          area_bh = buffer _Area:handle
          area_bf = area_bh:buffer-field( "_Area-ClusterSize" )
        no-error.
        tt_area.clstrSz = area_bf:buffer-value.
      end.

    tt_area.blkSzKB = _Area-blocksize / 1024.					/* _area-blocksize is in bytes		*/

  end.

  return.

end.


procedure getAreaVarInfo:

  define variable blksAlloc   as decimal no-undo.
  define variable blksVar     as decimal no-undo.
  define variable blksFree    as decimal no-undo.
  define variable extBlks     as decimal no-undo.
  define variable lastFixedSz as decimal no-undo.

  define variable recidExp    as integer no-undo.

  define variable x as decimal no-undo.

  x = ( 1024 * 1024 ).

  for each tt_area:

    assign
      blksAlloc = 0
      blksVar   = 0
    .

    /* _AreaExtent._Extent-size is in KB, it is the *planned* size from the .st file,
     *  variable extents are reported as zero.
     */

    find dictdb._AreaStatus no-lock where _AreaStatus-Id = tt_area.xid no-error.
    if not available _AreaStatus then next.

    lastFixedSz = 0.

    for each tt_AreaExtent no-lock where tt_AreaExtent.areaNum = tt_area.SANum:

      extBlks = ( extSize / tt_area.blkSzKB ).
      if extBlks = ? then extBlks = 0.

      if extType >= 4 and extType <= 7 then
        assign
          tt_area.hasVar = yes
          blksVar = max( extBlks, ( _AreaStatus-totblocks - blksAlloc ))
        .
       else
        assign
          lastFixedSz = ( extBlks * tt_area.blkSzKB ) / x
          blksAlloc = blksAlloc + extBlks
        .

    end.

    blksFree = _AreaStatus-Totblocks - _AreaStatus-Hiwater.
    if ( _AreaStatus-Freenum = ? ) then blksFree = blksFree + _AreaStatus-Freenum.

    if blksFree = ? then blksFree = _AreaStatus-totblocks.

    assign
      /* lastFixedSz     = ( extBlks   * tt_area.blkSzKB ) / x */
      tt_area.allocGB = ( blksAlloc * tt_area.blkSzKB ) / x
      tt_area.varGB   = ( blksVar   * tt_area.blkSzKB ) / x
      tt_area.freeGB  = ( blksFree  * tt_area.blkSzKB ) / x
      tt_area.totGB   = tt_area.allocGB + tt_area.varGB
    .

    assign
      tt_area.hiGB     = max( 0, ( tt_area.totGB - tt_area.freeGB ))
      tt_area.pctAlloc = (( tt_area.hiGB / tt_area.allocGB ) * 100 )
    .

    if tt_area.pctAlloc = ? then tt_area.pctAlloc = 0.

    if lastFixedSz <= 0 then
      tt_area.pctLastX = 0.			/* there is only a variable length extent - so pctLastX makes no sense	*/
     else
      do:
        if _AreaStatus-Hiwater >= blksAlloc then
          tt_area.pctLastX = 100.		/* we have overflowed and we are using the variable extent		*/
         else
          do:
            if tt_area.freeGB >= lastFixedSz then
              tt_area.pctLastX = 0.		/* we are not in the last fixed extent yet				*/
             else
              tt_area.pctLastX = ( 1 - ( tt_area.freeGB / lastFixedSz )) * 100.
          end.
      end.

    /* if we are within 80% of the 32/64 bit boundry that kbase 000081166 references:
     *   http://knowledgebase.progress.com/articles/Critical_Alert/Critical-Alert-Index-manager-defect-can-corrupt-large-indexes-where-dbkeys-straddle-32-64-bit-boundary
     */

    if ( tt_area.numIdxs > 0 ) then
      assign
        idx3264 = (( _areaStatus-hiWater / tt_area.rpb ) / exp( 2, 31 )) * 100
        xNote = xNote + "!" when idx3264 > 80
      .

    /* if a type1 area is approaching the 32 bit limit then its max area size may be an issue
     * t2 areas with 64 bit rowids seem unlikely to get there anytime soon -- but we may as well calculate it
     */
    if isDBFeatureActive( 9 ) and tt_area.clstrSz >= 8 then                   /* "64 Bit DBKEYS" (10.1B+) and Type II */
      recidExp = 63.
     else
      recidExp = 31.

    if ( SANum >= 6 ) and (( _areaStatus-Areaname matches "*After Image Area*" ) = no ) then
      assign
        areaMaxPct = (( _areaStatus-hiWater * tt_area.rpb ) / exp( 2, recidExp )) * 100
        xNote = xNote + "1" when areaMaxPct > 20
      .

  end.

  return.

end.


/* check ai extent status
 *
 */

/*
function chkai returns integer ( output ai_exts as integer, output ai_full as integer, output ai_empty as integer ):

  define variable ai_seq  as integer no-undo.
  define variable ai_busy as integer no-undo.

  assign
    ai_exts  = 0
    ai_busy  = 0
    ai_full  = 0
    ai_empty = 0
    ai_seq   = 0
  .

  for each _AreaStatus no-lock where ( _areaStatus-Areaname matches "*After Image Area*" ):

    ai_exts = ai_exts + 1.

    case aiInfo( input _AreaStatus-LastExtent, output ai_seq ):
      when "full"  then ai_full  = ai_full  + 1.
      when "empty" then ai_empty = ai_empty + 1.
      when "busy"  then ai_busy  = ai_exts.
    end.

  end.

  return ai_busy.

end.
 */


/* chkptNum
 *
 */

function chkptNum returns integer ( input-output oldbi as integer ):

  /* Adjust to align with checkpoint numbers -- if we can.  _Trans-counter is the
   * checkpoint# since the last "truncate bi".  Unfortunately the only way to map
   * from that number to _BfStatus-LastCkpNum involves starting a transaction in
   * mon-init().  And that is optional since ProTop shouldn't require the user to
   * write to a monitored database.  So if that option isn't enabled (chkp-base = ?)
   * we have to live with the fact that we can't map them.  Which means that we're
   * showing 0 at times when the real active checkpoint is probably something else.
   * That's not as bad as it sounds though because it should only happen when there
   * are no active transactions which means that, while we don't know what the
   * actual current checkpoint # is, we also only have one active checkpoint so
   * there's nothing to be excited about in these two numbers.  (The whole point
   * of this metric is to show growth in the number of active bi clusters.)
   *
   */

  define variable currbi as integer no-undo.

  assign
    oldbi  = 0
    currbi = 0
  .

  for each dictdb._Trans no-lock where _Trans-usrnum <> ? and _Trans-state <> "allocated":
    if _Trans-counter <> ? and _Trans-counter > 0 then
      do:
        currbi = max( currbi, _Trans-counter ).
        if oldbi = 0 or _Trans-counter < oldbi then oldbi = _Trans-counter.
      end.
  end.

  return currbi.

end.


/* is after-imaging enabled?
 *
 */

function isAIEnabled returns logical ():

  find dictdb._Logging no-lock no-error.
  if not available _logging then return no.

  return( if _Logging-AIBegin begins "-" then no else yes ).	         /* _Logging-AIJournal is apparently broken :( */

end.


/* is this db a replication source?
 *
 */

function isReplEnabled returns logical ():

    return isDBFeatureActive( 1 ).
    
end.


/* is this db a replication source?
 *
 */

define variable xReplSource as integer no-undo initial ?.

function isReplSource returns logical ():

  define variable ero as integer no-undo.

  if not isReplEnabled() then                                          /* replication is not available or not enabled */
    xReplSource = 0.													                         /* if disabled online we are no longer expecting to be a source	*/

  if isReplEnabled() then						                                   /* replication is enabled, is this a source or a target?	*/
    do:

      ero = ?.
      ero = integer( getStartUpX( "", "(-ERO)", "Enhanced Read-Only mode" )) no-error.				

      if       ero = 0 then xReplSource = 1.										       /* the source is NOT read-only */
       else if ero = 1 then xReplSource = 0.										       /* a target IS read-only */

      /* prior to 11.5 we cannot search for -ERO so check to see if a replServ is running, this isn't perfect: the VST disappears if replserv is stopped
       */

      if ero = ? then
        do:
          find first dictdb._repl-server no-lock no-error.
          if available( _repl-server ) then
            xReplSource = 1.
        /* else ...
         *
         * if we *ever* saw a replserv then assume that we should continue to see a replserv - so do NOT set xReplSource = 0
         * it will get reset to 0 above *if* replication is disabled online but NOT if a "terminate server" is issued or if the replserv crashes
         * starting without replserv will not be caught by this logic but a stopped or crashed replserv will
         *
         * and remember -- this is the pre-11.5 path, 11.5+ doesn't have this corner case
         */
        end.

    end.

  if xReplSource = ? or xReplSource = 0 then
    return false.
   else
    return true.

end.


/* is this db a replication target?
 *
 */

define variable xReplTarget as integer no-undo initial ?.

function isReplTarget returns logical ():

/*
 *  find first dictdb._repl-agent no-lock no-error.
 *
 *  return available( _repl-agent ).
 */

  define variable ero as integer no-undo.

  if not isDBFeatureActive( 1 ) then                                          /* replication is not available or not enabled */
    xReplTarget = 0.                                                          /* if disabled online we are no longer expecting to be a target	*/

  if isDBFeatureActive( 1 ) then                                              /* replication is enabled, is this a source or a target? */
    do:

      ero = ?.
      ero = integer( getStartUpX( "", "(-ERO)", "Enhanced Read-Only mode" )) no-error.				

      if       ero = 0 then xReplTarget = 0.										/* the source is NOT read-only					*/
       else if ero = 1 then xReplTarget = 1.										/* a target IS read-only					*/

      /* prior to 11.5 we cannot search for -ERO so check to see if a replServ is running, this isn't perfect: the VST disappears if replagent is stopped
       */

      if ero = ? then
        do:
          find first dictdb._repl-agent no-lock no-error.
          if available( _repl-agent ) then
            xReplTarget = 1.
        /* else ...
         *
         * if we *ever* saw a replagent then assume that we should continue to see a replagent - so do NOT set xReplTarget = 0
         * it will get reset to 0 above *if* replication is disabled online but NOT if a "terminate agent" is issued or if the replagent crashes
         * starting without replagent will not be caught by this logic but a stopped or crashed replagent will
         *
         * and remember -- this is the pre-11.5 path, 11.5+ doesn't have this corner case
         */
        end.

    end.

  if xReplTarget = ? or xReplTarget = 0 then
    return false.
   else
    return true.

end.


/* check to see if a backup is running
 *
 * also: if the backup completed recently (within the sample interval) consider it to be active and set backupStatus = yes
 */

function isBackupRunning returns logical ():

  define variable backupStatus as logical no-undo initial no.

  define variable fBackUp as integer no-undo initial ?.
  define variable iBackUp as integer no-undo initial ?.

  /* is an online backup running?
   */

  /* for each dictdb._Connect no-lock where _Connect-usr <> ? and _Connect-type = "BKUP": */

  for each dictdb._userStatus no-lock where _userStatus-operation <> ?:
    if _userStatus-operation = "online backup" then
      do:
        backupStatus = yes.
        leave.
      end.
  end.

  /* if the backup completed within the sample interval consider it to be active and set backupStatus = yes
   */

  find dictdb._mstrBlk no-lock no-error.

  if available _mstrBlk then
    do:

      fBackup = abs( interval( now, datetime( fixDateString( _mstrBlk-fbDate )), "seconds" )) no-error.
      iBackup = abs( interval( now, datetime( fixDateString( _mstrBlk-ibDate )), "seconds" )) no-error.

      if fBackup <> ? and fBackup < 300 then backupStatus = yes.
      if iBackup <> ? and iBackup < 300 then backupStatus = yes.

    end.

  return backupStatus.

end.


/* is this db a workgroup db?
 *
 */

function isWorkgroup returns logical ():

  return ( integer( trim( getStartUpX( "_Startup-spin", "(-spin)", "" ))) <= 1 ).

end.


/* try to get startup parameter values that are missing from VSTs from PROMON
 *
 * see dc/promon.txt for sample output
 *
 */

define variable startProMon  as character /* longchar */ /* no-undo */.

procedure loadProMonStartUp:

  define variable promonCmd  as character no-undo.
  define variable inLine     as character no-undo.

  if opsys = "unix" then
    promonCmd = "$DLC/bin/promon &1 < $PROTOP/etc/promon.startup 2> /dev/null".
   else
    promonCmd = "%DLC%~\bin~\promon &1 < %PROTOP%~\etc~\promon.startup 2> nul".

  startProMon = "".
  input stream inStrm through value( substitute( promonCmd, pdbname( 1 ))).
  repeat:
    import stream inStrm unformatted inLine.
    startProMon = startProMon + inLine + "~n".
  end.
  input stream inStrm close.

  return.

end.


/* gets a startup parameter from promon screen scraping
 *
 * p1 = parameter name in parenthesis
 * p2 = unique text
 *
 */

define variable promonLoaded as logical no-undo.

function getProMonParam returns character ( input p1 as character, input p2 as character ):

  define variable v as character no-undo.
  define variable t as character no-undo.

  define variable i as integer   no-undo.
  define variable n as integer   no-undo.

  if promonLoaded = no then
    do:
      run loadProMonStartUp.
      promonLoaded = yes.
    end.

  n = num-entries( startProMon, "~n" ).

  do i = 1 to n:

    t = entry( i, startProMon, "~n" ).

    if num-entries( t, ":" ) < 2 then next.

    if index( t, p1 ) > 0 then								/* try the parameter name first, i.e. "(-Mxs)"	*/
      do:
        v = trim( entry( 2, t, ":" )).
        leave.
      end.
     else if p2 <> "" and index( t, p2 ) > 0 then					/* try message text				*/
      do:
        v = trim( entry( 2, t, ":" )).
        leave.
      end.

  end.

  /*** message substitute( "getProMonParam looking for: [&1] [&2] in &3 lines; found: [&4]", p1, p2, n, v ). pause. ***/

  /* pica is specified in KB but reported in MB in the .lg and in PROMON - and reported in bytes in _dbParams, return bytes
   */

  if index( p1, "pica" ) > 0 then v = string( decimal( v ) * 1024 * 1024 ) no-error.

  return v.

end.


define variable hasDBParams  as logical no-undo.
define variable hasStartup   as logical no-undo.

define variable bdbp as handle no-undo.					/* buffer handle for _dbParams (preferred starting with 11.5)	*/
define variable bsu  as handle no-undo.					/* buffer handle for _startup (removed in oe12)			*/

procedure chkStartup:

  hasDBParams   = no.
  hasStartup    = no.

  find dictdb._File no-lock where _File-Name = "_DbParams" no-error.
  if available _File then
    do:
      hasDBParams = yes.
      create buffer bdbp for table "dictdb._DbParams".
    end.

  find dictdb._File no-lock where _File-Name = "_Startup" no-error.
  if available _File then
    do:
      hasStartup = yes.
      create buffer bsu for table "dictdb._Startup".
    end.

  return.

end.


/* get db startup parameters - try _dbParams 1st, _startup 2nd, and screen scraping last
 *
 * tt_configuration.excessSHM = integer( getStartUpX( "_Startup-MemOverflow", "(-Mxs)", "Excess shared memory" )) no-error.
 *
 * v  = _startUp field name
 * p1 = parameter name in parenthesis
 * p2 = unique text to find in .lg file or promon output
 *
 */

function getStartupX returns character ( input v as character, input p1 as character, input p2 as character ):

  define variable s  as character no-undo.
  define variable p  as character no-undo.
  define variable x  as decimal   no-undo.
  define variable f  as decimal   no-undo.

  define variable ok as logical   no-undo.

  define variable p1x as character no-undo case-sensitive.

  p1x = p1.

  /* if we have _dbParams go for that first!
   */

  if s = "" and hasDbParams = yes then
    do:
      ok = bdbp:find-first( substitute( 'where _dbParams-name = "&1"', trim( p1x, "()" )), no-lock ) no-error.
      if ok = yes then
        s = string( bdbp:buffer-field( "_dbParams-Value" ):buffer-value ) no-error.
      /*** message substitute( "looking for: [&1] in _dbParams; found [&2]", trim( p1x, "()" ), s ). pause. ***/
    end.

  /* use _startup if it is available
   */

  if s = "" and hasStartup then
    do:
      ok = bsu:find-first( '', no-lock ) no-error.
      if ok = yes then
        s = string( bsu:buffer-field( v ):buffer-value ) no-error.
    end.

  /* screen scraping -- last resort! :(
   */

  if s = "" then
    do:  

      /*** message substitute( "did not find: [&1] [&2] [&3]", v, p1x, p2 ). pause. ***/

      /* try promon output
       */

      s = trim( getProMonParam( p1x, p2 )).

      p = s.
      s = entry( 1, s, " " ).

      if num-entries( p, " " ) = 2 then
        do:
          if       entry( 2, p, " " ) begins "KB" then f = 1024.
           else if entry( 2, p, " " ) begins "MB" then f = 1024 * 1024.
           else if entry( 2, p, " " ) begins "GB" then f = 1024 * 1024 * 1024.
           else if entry( 2, p, " " ) begins "TB" then f = 1024 * 1024 * 1024 * 1024.
          if f > 0 then
            do:
              x = ?.
              x = decimal( s ) no-error.
              if x <> ? then s = string( x * f ).
            end.
        end.

    end.

  s = trim( s ).

  if s = "" or s = ? then s = "". 		/* used to be "N/A"	*/

  return s.

end.


function getUserInfo returns character ( usrNum as integer ):

  define variable lineNum  as integer   no-undo.
  define variable procName as character no-undo.
  define variable dbIdle   as character no-undo.

  define buffer xConnect for _Connect.

  find xConnect no-lock where _Connect-id = usrNum + 1 no-error.

  if available xConnect then
    return substitute( "&1:&2:&3 &4", _Connect-usr, _Connect-device, _Connect-PID, lastStatement( _Connect-Id, output lineNum, output procName, output dbIdle )).
   else
    return "".

end.


function snapshotLock returns logical ():

  define variable i        as integer   no-undo.
  define variable j        as integer   no-undo.
  define variable k        as integer   no-undo.
  define variable estart   as integer   no-undo.
  define variable lkscan   as integer   no-undo.
  define variable foundIt  as logical   no-undo.
  define variable xline    as character no-undo extent 16.

  define variable fastLock as logical no-undo.

  if decimal( substring( proversion, 1, index( proversion, "." ) + 1 )) >= 11.4 then fastlock = yes.

  empty temp-table tt_Lock.						/* defined in lib/protop.i				*/
  lkscan = 0.

  /*** find dictdb._DbStatus no-lock no-error. ***/

  estart = mtime.

  if mtime >= ( 86400000 - 10000 ) then return false.			/* don't bother to work out midnite roll overs		*/

  if fastlock then							/* 11.4+						*/
    do:

      /* 11.4+ have a much improved _Lock VST
       */

      for each dictdb._Lock where _Lock._Lock-usr <> ? and _Lock._Lock-recid <> ?:

        create tt_lock.
        buffer-copy _lock to tt_lock no-error.
        if index( _Lock._Lock-flags, "Q" ) > 0 then tt_lock.qflag = yes.
        lkscan = lkscan + 1.

        if ( dbgMode > 4 ) and ( session:batch = no ) and ( j = 0 ) and (( mtime - estart ) > 500 ) then	/* if the snapshot is fast don't bother the user	*/
          do:
            j = 1.
            message now "Making snapshot of _Lock:" lkscan ( mtime - estart ).
          end.

        /* yes, 11.4+ is _faster_ but it can still be painful if -L is very large
         */

        if (( mtime - estart ) >= 5000 ) or ( lkScan >= pt_lktbllim ) then	/* default pt_lktbllim = 10,000				*/
          do:

            if ( dbgMode > 4 ) or ( session:batch = no ) then
              message now "_Lock scan is too slow, abandonded, scan:" lkScan "  mtime:" ( mtime - estart ).

            return false.

          end.
  
      end.

    end.
   else									/* do it the old, slow, way but limit our effort	*/
    do:

      if opsys begins "win" then
        input stream inStrm through value( "promon " + pdbname( 1 ) + " < etc/promon.dump_locks 2>NUL" ).
       else
        input stream inStrm through value( "promon " + pdbname( 1 ) + " < etc/promon.dump_locks 2> /dev/null" ).

      repeat:

        xline = "".

        import stream inStrm xline.

        if xline[1] = "" then
          next.
         else if foundIt = yes and xline[1] = "OpenEdge" and xline[2] = "Monitor" and xline[3] = "Session" and xline[4] = "End." then
          leave.

        if j = 1 then	/* oe10	*/
          do:

            if xline[4] <> "REC" then next.

            i = i + 1.

            create tt_lock.
            assign
              tt_lock._lock-id    = i
              tt_lock._lock-usr   = integer( xline[1] )
              tt_lock._lock-recid = int64(   xline[6] )
              tt_lock._lock-table = integer( xline[5] )
            .

            lkscan = lkscan + 1.

            do k = 7 to 16:
              if index( xline[k], "Q" ) > 0 then tt_lock.qflag = yes.
              if tt_lock.qflag then leave.
            end.

          end.
         else if j = 2 then	/* oe11 */
          do:

            if xline[5] <> "REC" then next.

            i = i + 1.

            create tt_lock.
            assign
              tt_lock._lock-id    = i
              tt_lock._lock-usr   = integer( xline[1] )
              tt_lock._lock-recid = int64(   xline[7] )
              tt_lock._lock-table = integer( entry( 1, xline[6], ":" ))
            .

            do k = 8 to 16:
              if index( xline[k], "Q" ) > 0 then tt_lock.qflag = yes.
              if tt_lock.qflag then leave.
            end.

          end.

        if       xline[1] = "Usr"     and xline[2] = "Name" then j = 1.
         else if xline[1] = "Usr:Ten" and xline[2] = "Name" then j = 2.

        if (( mtime - estart ) >= 5000 ) or ( lkScan >= pt_lktbllim ) then		/* default pt_lktbllim = 10,000				*/
          do:

            input stream inStrm close.

            if ( dbgMode > 3 ) or ( session:batch = no ) then
              message now "_Lock scan is too slow, abandonded, scan:" lkScan "  mtime:" ( mtime - estart ).

            return false.

          end.

      end.

      input stream inStrm close.

    end.

  if false then
    do:
      output to value( "log/locks.txt" ).
      message now i "locks found in promon".
      for each tt_lock:
        display tt_lock.
      end.
      output close.
    end.

  return true.

end.

/* end vstlib.p */
