#!/bin/sh
#
# ${PROTOP}/bin/quiet.sh friendlyName on|off
#

. #ENVPT3#/bin/protopenv

# UN-comment for DEBUG
#ECHO=echo

TMPDIR=${TMPDIR-/tmp}
LOGDIR=${LOGDIR-/tmp}

MNAME="quiet"

cd ${PROTOP}

if [ ! -d ${TMPDIR} ]
then
        mkdir ${TMPDIR}
fi

if [ ! -d ${LOGDIR} ]
then
        mkdir ${LOGDIR}
fi

errmsg()
{
        echo ""
        echo "${1}"
        echo ""
}

# save ${1} and shift the command args so that we can easily pass along $2 etc
#
# P1 is a "friendly name" in etc/dblist.cfg
#

P1=${1}

if [ -z "${P1}" ]
then
        errmsg "Please specify a friendly name in etc/dblist.cfg."
        exit 1
fi

shift

P2=${1}

if [ "${P2}" != "on" ] && [ "${P2}" != "off" ]
then
        errmsg "Please specify "on" or "off" as parameter 2. "
        exit 1
fi


XDB=

if [ -f ${PROTOP}/etc/dblist.cfg ]      # search using "pipe" as the delimiter
then
        XDB=`grep "^${P1}|" ${PROTOP}/etc/dblist.cfg | awk -F "|" '{print $2}' 2>/dev/null`
fi

# if XDB has a value then we found a friendly name and the path is in XDB
#

        if [ -z "${XDB}" ]                                      # no friendly name?
        then
                XDB=${P1}                                       # take ${P1} as the path name
        fi

if [ ! -f ${XDB}.db ]
then
        errmsg "${XDB}.db does not exist."
        exit 1
fi

loop=0
MaxLoop=10

## Off
if [ "${P2}" = "off" ]
then
  $ECHO proquiet $XDB -C disable;RC=$?
  if [ $RC != 0 ]
  then
    errmsg "proquiet  $XDB -C disable failed with exit code $RC"
    exit 1
  else
    echo "proquiet  $XDB -C disable completed successfully."
    exit 0
  fi
fi


# On

# Search string:
# [2017/12/14@14:12:58.251-0500] P-762 T-140469713327936 I BROKER 0: (5583) Quiet point has been enabled by the broker.


while true
do

  if (( loop > MaxLoop ))
  then
    echo "FAIL: too many attempts $loop GT $MaxLoop"
    exit 1
  fi

  BeforeLines=$(wc -l ${XDB}.lg | awk '{print $1}')
  $ECHO proquiet $XDB -C enable;RC=$?
  if [ $RC = 0 ]
  then

    echo "proquiet command returned 0. Checking for (5583) message"

    AfterLines=$(wc -l ${XDB}.lg | awk '{print $1}')
    (( BetweenLines = AfterLines - BeforeLines ))

    tail -$BetweenLines ${XDB}.lg | grep -q "(5583)";RC=$?

    if [ $RC = 0 ]
    then
      echo "SUCCESS: 5583 message found"
      break
    fi
  else
    echo "Proquiet enable failed"
  fi

    ((loop = loop + 1))
    sleep 1

done

exit 0

